use std::{fs, io, path::PathBuf};

use base64::prelude::*;
use openssl::{
    hash::MessageDigest,
    pkey::{PKey, Private, Public},
    sign::Signer,
};

pub struct Key {
    pub(crate) privkey: PKey<Private>,
    pub(crate) pubkey: PKey<Public>,
    pub(crate) key_id: String,
}

impl Key {
    pub fn from_files(
        privkey_file: PathBuf,
        pubkey_file: PathBuf,
        key_id: String,
    ) -> Result<Self, io::Error> {
        let privkey_file = fs::read(privkey_file)?;
        let pubkey_file = fs::read(pubkey_file)?;

        Ok(Self::from_bytes(&privkey_file, &pubkey_file, key_id))
    }

    pub fn from_bytes(privkey: &[u8], pubkey: &[u8], key_id: String) -> Self {
        let privkey = PKey::private_key_from_pem(privkey).expect("Failed decoding private key pem");
        let pubkey = PKey::public_key_from_pem(pubkey).expect("Failed decoding public key pem");

        Self {
            privkey,
            pubkey,
            key_id,
        }
    }

    pub fn sign(&self, bytes: &[u8]) -> String {
        let mut signer = Signer::new(MessageDigest::sha256(), &self.privkey).unwrap();
        signer.update(bytes).unwrap();
        BASE64_STANDARD.encode(signer.sign_to_vec().unwrap())
    }
}
